@@ -1,12 +1,18 @@
 #!/bin/sh
 set -ex
 
+sudo add-apt-repository ppa:kirillshkrogalev/ffmpeg-next -y
 sudo apt-get -qq update
 sudo apt-get install -y \
         build-essential \
         checkinstall \
         cmake \
         libasound2-dev \
+        libavcodec-ffmpeg-dev \
+        libavdevice-ffmpeg-dev \
+        libavfilter-ffmpeg-dev \
+        libavformat-ffmpeg-dev \
+        libavutil-ffmpeg-dev \
         libcurl4-openssl-dev \
         libfdk-aac-dev \
         libfontconfig-dev \
@@ -14,9 +20,12 @@ sudo apt-get install -y \
         libgl1-mesa-dev \
         libjack-jackd2-dev \
         libjansson-dev \
+        libluajit-5.1-dev \
         libpulse-dev \
         libqt5x11extras5-dev \
         libspeexdsp-dev \
+        libswresample-ffmpeg-dev \
+        libswscale-ffmpeg-dev \
         libudev-dev \
         libv4l-dev \
         libvlc-dev \
@@ -27,14 +36,6 @@ sudo apt-get install -y \
         libxcomposite-dev \
         libxinerama-dev \
         pkg-config \
+        python3-dev \
         qtbase5-dev \
-        yasm \
-        zlib1g-dev
-
-# FFmpeg
-cd ..
-git clone --depth 1 git://source.ffmpeg.org/ffmpeg.git
-cd ffmpeg
-./configure --enable-shared
-make -j2
-sudo make install
+        swig
