@@ -24,8 +24,8 @@ static AVCodec *find_hardware_decoder(enum AVCodecID id)
 
 	while (hwa) {
 		if (hwa->id == id) {
-			if (hwa->pix_fmt == AV_PIX_FMT_VDA_VLD ||
-			    hwa->pix_fmt == AV_PIX_FMT_DXVA2_VLD ||
+			if (hwa->pix_fmt == AV_PIX_FMT_VDTOOL ||
+				hwa->pix_fmt == AV_PIX_FMT_DXVA2_VLD ||
 			    hwa->pix_fmt == AV_PIX_FMT_VAAPI_VLD) {
 				c = avcodec_find_decoder_by_name(hwa->name);
 				if (c)
@@ -142,8 +142,8 @@ bool mp_decode_init(mp_media_t *m, enum AVMediaType type, bool hw)
 		return false;
 	}
 
-	if (d->codec->capabilities & CODEC_CAP_TRUNCATED)
-		d->decoder->flags |= CODEC_FLAG_TRUNCATED;
+	if (d->codec->capabilities & CODEC_CAP_TRUNC)
+		d->decoder->flags |= CODEC_FLAG_TRUNC;
 	return true;
 }
 
@@ -167,14 +167,16 @@ void mp_decode_free(struct mp_decode *d)
 	circlebuf_free(&d->packets);
 
 	if (d->decoder) {
-		avcodec_close(d->decoder);
 #if LIBAVFORMAT_VERSION_INT >= AV_VERSION_INT(57, 40, 101)
-		av_free(d->decoder);
+		avcodec_free_context(&d->decoder);
+#else
+		avcodec_close(d->decoder);
 #endif
 	}
-
-	if (d->frame)
+	if (d->frame) {
+		av_frame_unref(d->frame);
 		av_free(d->frame);
+	}
 
 	memset(d, 0, sizeof(*d));
 }
@@ -279,18 +281,6 @@ bool mp_decode_next(struct mp_decode *d)
 			}
 		}
 
-		if (!d->audio && d->m->is_network && !d->got_first_keyframe) {
-			if (d->pkt.flags & AV_PKT_FLAG_KEY) {
-				d->got_first_keyframe = true;
-			} else {
-				av_packet_unref(&d->orig_pkt);
-				av_init_packet(&d->orig_pkt);
-				av_init_packet(&d->pkt);
-				d->packet_pending = false;
-				return true;
-			}
-		}
-
 		ret = decode_packet(d, &got_frame);
 
 		if (!got_frame && ret == 0) {
@@ -332,9 +322,13 @@ bool mp_decode_next(struct mp_decode *d)
 	if (d->frame_ready) {
 		int64_t last_pts = d->frame_pts;
 
-		d->frame_pts = av_rescale_q(d->frame->best_effort_timestamp,
-				d->stream->time_base,
-				(AVRational){1, 1000000000});
+		if (d->frame->best_effort_timestamp == AV_NOPTS_VALUE)
+			d->frame_pts = d->next_pts;
+		else
+			d->frame_pts = av_rescale_q(
+					d->frame->best_effort_timestamp,
+					d->stream->time_base,
+					(AVRational){1, 1000000000});
 
 		int64_t duration = d->frame->pkt_duration;
 		if (!duration)
@@ -343,6 +337,16 @@ bool mp_decode_next(struct mp_decode *d)
 			duration = av_rescale_q(duration,
 					d->stream->time_base,
 					(AVRational){1, 1000000000});
+
+		if (d->m->speed != 100) {
+			d->frame_pts = av_rescale_q(d->frame_pts,
+					(AVRational){1, d->m->speed},
+					(AVRational){1, 100});
+			duration = av_rescale_q(duration,
+					(AVRational){1, d->m->speed},
+					(AVRational){1, 100});
+		}
+
 		d->last_duration = duration;
 		d->next_pts = d->frame_pts + duration;
 	}
