@@ -44,6 +44,11 @@ static inline int64_t packet_dts_usec(struct encoder_packet *packet)
 	return packet->dts * MICROSECOND_DEN / packet->timebase_den;
 }
 
+struct tick_callback {
+	void (*tick)(void *param, float seconds);
+	void *param;
+};
+
 struct draw_callback {
 	void (*draw)(void *param, uint32_t cx, uint32_t cy);
 	void *param;
@@ -82,6 +87,7 @@ struct obs_module {
 
 	bool        (*load)(void);
 	void        (*unload)(void);
+	void        (*post_load)(void);
 	void        (*set_locale)(const char *locale);
 	void        (*free_locale)(void);
 	uint32_t    (*ver)(void);
@@ -242,6 +248,7 @@ struct obs_core_video {
 	gs_samplerstate_t               *point_sampler;
 	gs_stagesurf_t                  *mapped_surface;
 	int                             cur_texture;
+	long                            raw_active;
 
 	uint64_t                        video_time;
 	uint64_t                        video_avg_frame_time_ns;
@@ -318,11 +325,14 @@ struct obs_core_data {
 	pthread_mutex_t                 audio_sources_mutex;
 	pthread_mutex_t                 draw_callbacks_mutex;
 	DARRAY(struct draw_callback)    draw_callbacks;
+	DARRAY(struct tick_callback)    tick_callbacks;
 
 	struct obs_view                 main_view;
 
 	long long                       unnamed_index;
 
+	obs_data_t                      *private_data;
+
 	volatile bool                   valid;
 };
 
@@ -393,7 +403,7 @@ struct obs_core {
 
 extern struct obs_core *obs;
 
-extern void *obs_video_thread(void *param);
+extern void *obs_graphics_thread(void *param);
 
 extern gs_effect_t *obs_load_effect(gs_effect_t **effect, const char *file);
 
@@ -401,6 +411,13 @@ extern bool audio_callback(void *param,
 		uint64_t start_ts_in, uint64_t end_ts_in, uint64_t *out_ts,
 		uint32_t mixers, struct audio_output_data *mixes);
 
+extern void start_raw_video(video_t *video,
+		const struct video_scale_info *conversion,
+		void (*callback)(void *param, struct video_data *frame),
+		void *param);
+extern void stop_raw_video(video_t *video,
+		void (*callback)(void *param, struct video_data *frame),
+		void *param);
 
 /* ------------------------------------------------------------------------- */
 /* obs shared context data */
@@ -609,6 +626,7 @@ struct obs_source {
 	bool                            async_active;
 	bool                            async_update_texture;
 	bool                            async_unbuffered;
+	bool                            async_decoupled;
 	struct obs_source_frame         *async_preload_frame;
 	DARRAY(struct async_frame)      async_cache;
 	DARRAY(struct obs_source_frame*)async_frames;
@@ -679,16 +697,15 @@ struct obs_source {
 
 	struct audio_monitor            *monitor;
 	enum obs_monitoring_type        monitoring_type;
+
+	obs_data_t                      *private_settings;
 };
 
-extern const struct obs_source_info *get_source_info(const char *id);
+extern struct obs_source_info *get_source_info(const char *id);
 extern bool obs_source_init_context(struct obs_source *source,
 		obs_data_t *settings, const char *name,
 		obs_data_t *hotkey_data, bool private);
 
-extern void obs_source_save(obs_source_t *source);
-extern void obs_source_load(obs_source_t *source);
-
 extern bool obs_transition_init(obs_source_t *transition);
 extern void obs_transition_free(obs_source_t *transition);
 extern void obs_transition_tick(obs_source_t *transition);
