@@ -15,6 +15,7 @@
     along with this program.  If not, see <http://www.gnu.org/licenses/>.
 ******************************************************************************/
 
+#include "util/windows/win-registry.h"
 #include "util/windows/win-version.h"
 #include "util/platform.h"
 #include "util/dstr.h"
@@ -22,6 +23,8 @@
 #include "obs-internal.h"
 
 #include <windows.h>
+#include <wscapi.h>
+#include <iwscapi.h>
 
 static uint32_t win_ver = 0;
 
@@ -189,6 +192,187 @@ static void log_aero(void)
 			aeroMessage);
 }
 
+#define WIN10_GAME_BAR_REG_KEY \
+		L"Software\\Microsoft\\Windows\\CurrentVersion\\GameDVR"
+#define WIN10_GAME_DVR_POLICY_REG_KEY \
+		L"SOFTWARE\\Policies\\Microsoft\\Windows\\GameDVR"
+#define WIN10_GAME_DVR_REG_KEY L"System\\GameConfigStore"
+#define WIN10_GAME_MODE_REG_KEY L"Software\\Microsoft\\GameBar"
+
+static void log_gaming_features(void)
+{
+	if (win_ver < 0xA00)
+		return;
+
+	struct reg_dword game_bar_enabled;
+	struct reg_dword game_dvr_allowed;
+	struct reg_dword game_dvr_enabled;
+	struct reg_dword game_dvr_bg_recording;
+	struct reg_dword game_mode_enabled;
+
+	get_reg_dword(HKEY_CURRENT_USER, WIN10_GAME_BAR_REG_KEY,
+			L"AppCaptureEnabled", &game_bar_enabled);
+	get_reg_dword(HKEY_CURRENT_USER, WIN10_GAME_DVR_POLICY_REG_KEY,
+			L"AllowGameDVR", &game_dvr_allowed);
+	get_reg_dword(HKEY_CURRENT_USER, WIN10_GAME_DVR_REG_KEY,
+			L"GameDVR_Enabled", &game_dvr_enabled);
+	get_reg_dword(HKEY_CURRENT_USER, WIN10_GAME_BAR_REG_KEY,
+			L"HistoricalCaptureEnabled", &game_dvr_bg_recording);
+	get_reg_dword(HKEY_CURRENT_USER, WIN10_GAME_MODE_REG_KEY,
+			L"AllowAutoGameMode", &game_mode_enabled);
+
+	blog(LOG_INFO, "Windows 10 Gaming Features:");
+	if (game_bar_enabled.status == ERROR_SUCCESS) {
+		blog(LOG_INFO, "\tGame Bar: %s",
+			(bool)game_bar_enabled.return_value ? "On" : "Off");
+	}
+
+	if (game_dvr_allowed.status == ERROR_SUCCESS) {
+		blog(LOG_INFO, "\tGame DVR Allowed: %s",
+			(bool)game_dvr_allowed.return_value ? "Yes" : "No");
+	}
+
+	if (game_dvr_enabled.status == ERROR_SUCCESS) {
+		blog(LOG_INFO, "\tGame DVR: %s",
+			(bool)game_dvr_enabled.return_value ? "On" : "Off");
+	}
+
+	if (game_dvr_bg_recording.status == ERROR_SUCCESS) {
+		blog(LOG_INFO, "\tGame DVR Background Recording: %s",
+			(bool)game_dvr_bg_recording.return_value ? "On" :
+			"Off");
+	}
+
+	if (game_mode_enabled.status == ERROR_SUCCESS) {
+		blog(LOG_INFO, "\tGame Mode: %s",
+			(bool)game_mode_enabled.return_value ? "On" : "Off");
+	}
+}
+
+static const char *get_str_for_state(int state)
+{
+	switch (state) {
+	case WSC_SECURITY_PRODUCT_STATE_ON:
+		return "enabled";
+	case WSC_SECURITY_PRODUCT_STATE_OFF:
+		return "disabled";
+	case WSC_SECURITY_PRODUCT_STATE_SNOOZED:
+		return "temporarily disabled";
+	case WSC_SECURITY_PRODUCT_STATE_EXPIRED:
+		return "expired";
+	default:
+		return "unknown";
+	}
+}
+
+static const char *get_str_for_type(int type)
+{
+	switch (type) {
+	case WSC_SECURITY_PROVIDER_ANTIVIRUS:
+		return "AV";
+	case WSC_SECURITY_PROVIDER_FIREWALL:
+		return "FW";
+	case WSC_SECURITY_PROVIDER_ANTISPYWARE:
+		return "ASW";
+	default:
+		return "unknown";
+	}
+}
+
+static void log_security_products_by_type(IWSCProductList *prod_list, int type)
+{
+	HRESULT hr;
+	LONG count = 0;
+	IWscProduct *prod;
+	BSTR name;
+	WSC_SECURITY_PRODUCT_STATE prod_state;
+
+	hr = prod_list->lpVtbl->Initialize(prod_list, type);
+
+	if (FAILED(hr))
+		return;
+
+	hr = prod_list->lpVtbl->get_Count(prod_list, &count);
+	if (FAILED(hr)) {
+		prod_list->lpVtbl->Release(prod_list);
+		return;
+	}
+
+	for (int i = 0; i < count; i++) {
+		hr = prod_list->lpVtbl->get_Item(prod_list, i, &prod);
+		if (FAILED(hr))
+			continue;
+
+		hr = prod->lpVtbl->get_ProductName(prod, &name);
+		if (FAILED(hr))
+			continue;
+
+		hr = prod->lpVtbl->get_ProductState(prod, &prod_state);
+		if (FAILED(hr)) {
+			SysFreeString(name);
+			continue;
+		}
+
+		blog(LOG_INFO, "\t%S: %s (%s)", name,
+				get_str_for_state(prod_state),
+				get_str_for_type(type));
+
+		SysFreeString(name);
+		prod->lpVtbl->Release(prod);
+	}
+
+	prod_list->lpVtbl->Release(prod_list);
+}
+
+static void log_security_products(void)
+{
+	IWSCProductList *prod_list = NULL;
+	HMODULE h_wsc;
+	HRESULT hr;
+
+	/* We load the DLL rather than import wcsapi.lib because the clsid /
+	 * iid only exists on Windows 8 or higher. */
+
+	h_wsc = LoadLibraryW(L"wscapi.dll");
+	if (!h_wsc)
+		return;
+
+	const CLSID *prod_list_clsid =
+		(const CLSID *)GetProcAddress(h_wsc, "CLSID_WSCProductList");
+	const IID *prod_list_iid =
+		(const IID *)GetProcAddress(h_wsc, "IID_IWSCProductList");
+
+	if (prod_list_clsid && prod_list_iid) {
+		blog(LOG_INFO, "Sec. Software Status:");
+
+		hr = CoCreateInstance(prod_list_clsid, NULL,
+				CLSCTX_INPROC_SERVER, prod_list_iid,
+				&prod_list);
+		if (!FAILED(hr)) {
+			log_security_products_by_type(prod_list,
+					WSC_SECURITY_PROVIDER_ANTIVIRUS);
+		}
+
+		hr = CoCreateInstance(prod_list_clsid, NULL,
+				CLSCTX_INPROC_SERVER, prod_list_iid,
+				&prod_list);
+		if (!FAILED(hr)) {
+			log_security_products_by_type(prod_list,
+					WSC_SECURITY_PROVIDER_FIREWALL);
+		}
+
+		hr = CoCreateInstance(prod_list_clsid, NULL,
+				CLSCTX_INPROC_SERVER, prod_list_iid,
+				&prod_list);
+		if (!FAILED(hr)) {
+			log_security_products_by_type(prod_list,
+					WSC_SECURITY_PROVIDER_ANTISPYWARE);
+		}
+	}
+
+	FreeLibrary(h_wsc);
+}
+
 void log_system_info(void)
 {
 	struct win_version_info ver;
@@ -202,6 +386,8 @@ void log_system_info(void)
 	log_windows_version();
 	log_admin_status();
 	log_aero();
+	log_gaming_features();
+	log_security_products();
 }
 
 
@@ -328,12 +514,16 @@ static int get_virtual_key(obs_key_t key)
 	case OBS_KEY_BRACKETRIGHT: return VK_OEM_6;
 	case OBS_KEY_ASCIITILDE: return VK_OEM_3;
 
+	case OBS_KEY_HENKAN: return VK_CONVERT;
+	case OBS_KEY_MUHENKAN: return VK_NONCONVERT;
 	case OBS_KEY_KANJI: return VK_KANJI;
 	case OBS_KEY_TOUROKU: return VK_OEM_FJ_TOUROKU;
 	case OBS_KEY_MASSYO: return VK_OEM_FJ_MASSHOU;
 
 	case OBS_KEY_HANGUL: return VK_HANGUL;
 
+	case OBS_KEY_BACKSLASH_RT102: return VK_OEM_102;
+
 	case OBS_KEY_MOUSE1: return VK_LBUTTON;
 	case OBS_KEY_MOUSE2: return VK_RBUTTON;
 	case OBS_KEY_MOUSE3: return VK_MBUTTON;
@@ -598,6 +788,13 @@ void reset_win32_symbol_paths(void)
 	da_free(paths);
 }
 
+extern void initialize_crash_handler(void);
+
+void obs_init_win32_crash_handler(void)
+{
+	initialize_crash_handler();
+}
+
 void initialize_com(void)
 {
 	CoInitializeEx(0, COINIT_MULTITHREADED);
