@@ -31,9 +31,6 @@
 #define FF_BLOG(level, format, ...) \
 	FF_LOG_S(s->source, level, format, ##__VA_ARGS__)
 
-static bool video_frame(struct ff_frame *frame, void *opaque);
-static bool video_format(AVCodecContext *codec_context, void *opaque);
-
 struct ffmpeg_source {
 	mp_media_t media;
 	bool media_valid;
@@ -51,12 +48,15 @@ struct ffmpeg_source {
 
 	char *input;
 	char *input_format;
+	int buffering_mb;
+	int speed_percent;
 	bool is_looping;
 	bool is_local_file;
 	bool is_hw_decoding;
 	bool is_clear_on_media_end;
 	bool restart_on_activate;
 	bool close_when_inactive;
+	bool seekable;
 };
 
 static bool is_local_file_modified(obs_properties_t *props,
@@ -70,12 +70,18 @@ static bool is_local_file_modified(obs_properties_t *props,
 			"input_format");
 	obs_property_t *local_file = obs_properties_get(props, "local_file");
 	obs_property_t *looping = obs_properties_get(props, "looping");
+	obs_property_t *buffering = obs_properties_get(props, "buffering_mb");
 	obs_property_t *close = obs_properties_get(props, "close_when_inactive");
+	obs_property_t *seekable = obs_properties_get(props, "seekable");
+	obs_property_t *speed = obs_properties_get(props, "speed_percent");
 	obs_property_set_visible(input, !enabled);
 	obs_property_set_visible(input_format, !enabled);
+	obs_property_set_visible(buffering, !enabled);
 	obs_property_set_visible(close, enabled);
 	obs_property_set_visible(local_file, enabled);
 	obs_property_set_visible(looping, enabled);
+	obs_property_set_visible(speed, enabled);
+	obs_property_set_visible(seekable, !enabled);
 
 	return true;
 }
@@ -89,6 +95,8 @@ static void ffmpeg_source_defaults(obs_data_t *settings)
 #if defined(_WIN32)
 	obs_data_set_default_bool(settings, "hw_decode", true);
 #endif
+	obs_data_set_default_int(settings, "buffering_mb", 2);
+	obs_data_set_default_int(settings, "speed_percent", 100);
 }
 
 static const char *media_filter =
@@ -167,6 +175,9 @@ static obs_properties_t *ffmpeg_source_getproperties(void *data)
 	obs_property_set_long_description(prop,
 			obs_module_text("CloseFileWhenInactive.ToolTip"));
 
+	obs_properties_add_int_slider(props, "speed_percent",
+			obs_module_text("SpeedPercentage"), 1, 200, 1);
+
 	prop = obs_properties_add_list(props, "color_range",
 			obs_module_text("ColorRange"), OBS_COMBO_TYPE_LIST,
 			OBS_COMBO_FORMAT_INT);
@@ -177,6 +188,8 @@ static obs_properties_t *ffmpeg_source_getproperties(void *data)
 	obs_property_list_add_int(prop, obs_module_text("ColorRange.Full"),
 			VIDEO_RANGE_FULL);
 
+	obs_properties_add_bool(props, "seekable", obs_module_text("Seekable"));
+
 	return props;
 }
 
@@ -187,6 +200,7 @@ static void dump_source_info(struct ffmpeg_source *s, const char *input,
 			"settings:\n"
 			"\tinput:                   %s\n"
 			"\tinput_format:            %s\n"
+			"\tspeed:                   %d\n"
 			"\tis_looping:              %s\n"
 			"\tis_hw_decoding:          %s\n"
 			"\tis_clear_on_media_end:   %s\n"
@@ -194,6 +208,7 @@ static void dump_source_info(struct ffmpeg_source *s, const char *input,
 			"\tclose_when_inactive:     %s",
 			input ? input : "(null)",
 			input_format ? input_format : "(null)",
+			s->speed_percent,
 			s->is_looping ? "yes" : "no",
 			s->is_hw_decoding ? "yes" : "no",
 			s->is_clear_on_media_end ? "yes" : "no",
@@ -210,7 +225,11 @@ static void get_frame(void *opaque, struct obs_source_frame *f)
 static void preload_frame(void *opaque, struct obs_source_frame *f)
 {
 	struct ffmpeg_source *s = opaque;
-	obs_source_preload_video(s->source, f);
+	if (s->close_when_inactive)
+		return;
+
+	if (s->is_clear_on_media_end || s->is_looping)
+		obs_source_preload_video(s->source, f);
 }
 
 static void get_audio(void *opaque, struct obs_source_audio *a)
@@ -224,22 +243,37 @@ static void media_stopped(void *opaque)
 	struct ffmpeg_source *s = opaque;
 	if (s->is_clear_on_media_end) {
 		obs_source_output_video(s->source, NULL);
-		if (s->close_when_inactive)
+		if (s->close_when_inactive && s->media_valid)
 			s->destroy_media = true;
 	}
 }
 
 static void ffmpeg_source_open(struct ffmpeg_source *s)
 {
-	if (s->input && *s->input)
-		s->media_valid = mp_media_init(&s->media,
-				s->input, s->input_format,
-				s, get_frame, get_audio, media_stopped,
-				preload_frame, s->is_hw_decoding, s->range);
+	if (s->input && *s->input) {
+		struct mp_media_info info = {
+			.opaque = s,
+			.v_cb = get_frame,
+			.v_preload_cb = preload_frame,
+			.a_cb = get_audio,
+			.stop_cb = media_stopped,
+			.path = s->input,
+			.format = s->input_format,
+			.buffering = s->buffering_mb * 1024 * 1024,
+			.speed = s->speed_percent,
+			.force_range = s->range,
+			.hardware_decoding = s->is_hw_decoding,
+			.is_local_file = s->is_local_file || s->seekable
+		};
+
+		s->media_valid = mp_media_init(&s->media, &info);
+	}
 }
 
 static void ffmpeg_source_tick(void *data, float seconds)
 {
+	UNUSED_PARAMETER(seconds);
+
 	struct ffmpeg_source *s = data;
 	if (s->destroy_media) {
 		if (s->media_valid) {
@@ -303,7 +337,13 @@ static void ffmpeg_source_update(void *data, obs_data_t *settings)
 			"restart_on_activate");
 	s->range = (enum video_range_type)obs_data_get_int(settings,
 			"color_range");
+	s->buffering_mb = (int)obs_data_get_int(settings, "buffering_mb");
+	s->speed_percent = (int)obs_data_get_int(settings, "speed_percent");
 	s->is_local_file = is_local_file;
+	s->seekable = obs_data_get_bool(settings, "seekable");
+
+	if (s->speed_percent < 1 || s->speed_percent > 200)
+		s->speed_percent = 100;
 
 	if (s->media_valid) {
 		mp_media_free(&s->media);
@@ -325,7 +365,7 @@ static const char *ffmpeg_source_getname(void *unused)
 	return obs_module_text("FFMpegSource");
 }
 
-static bool restart_hotkey(void *data, obs_hotkey_id id,
+static void restart_hotkey(void *data, obs_hotkey_id id,
 		obs_hotkey_t *hotkey, bool pressed)
 {
 	UNUSED_PARAMETER(id);
@@ -335,7 +375,6 @@ static bool restart_hotkey(void *data, obs_hotkey_id id,
 	struct ffmpeg_source *s = data;
 	if (obs_source_active(s->source))
 		ffmpeg_source_start(s);
-	return true;
 }
 
 static void restart_proc(void *data, calldata_t *cd)
@@ -344,6 +383,53 @@ static void restart_proc(void *data, calldata_t *cd)
 	UNUSED_PARAMETER(cd);
 }
 
+static void get_duration(void *data, calldata_t *cd)
+{
+	struct ffmpeg_source *s = data;
+	int64_t dur = 0;
+	if (s->media.fmt)
+		dur = s->media.fmt->duration;
+
+	calldata_set_int(cd, "duration", dur * 1000);
+}
+
+static void get_nb_frames(void *data, calldata_t *cd)
+{
+	struct ffmpeg_source *s = data;
+	int64_t frames = 0;
+
+	if (!s->media.fmt) {
+		calldata_set_int(cd, "num_frames", frames);
+		return;
+	}
+
+	int video_stream_index = av_find_best_stream(s->media.fmt,
+			AVMEDIA_TYPE_VIDEO, -1, -1, NULL, 0);
+
+	if (video_stream_index < 0) {
+		FF_BLOG(LOG_WARNING, "Getting number of frames failed: No "
+				"video stream in media file!");
+		calldata_set_int(cd, "num_frames", frames);
+		return;
+	}
+
+	AVStream *stream = s->media.fmt->streams[video_stream_index];
+
+	if (stream->nb_frames > 0) {
+		frames = stream->nb_frames;
+	} else {
+		FF_BLOG(LOG_DEBUG, "nb_frames not set, estimating using frame "
+				"rate and duration");
+		AVRational avg_frame_rate = stream->avg_frame_rate;
+		frames = (int64_t)ceil((double)s->media.fmt->duration /
+				(double)AV_TIME_BASE *
+				(double)avg_frame_rate.num /
+				(double)avg_frame_rate.den);
+	}
+
+	calldata_set_int(cd, "num_frames", frames);
+}
+
 static void *ffmpeg_source_create(obs_data_t *settings, obs_source_t *source)
 {
 	UNUSED_PARAMETER(settings);
@@ -358,6 +444,10 @@ static void *ffmpeg_source_create(obs_data_t *settings, obs_source_t *source)
 
 	proc_handler_t *ph = obs_source_get_proc_handler(source);
 	proc_handler_add(ph, "void restart()", restart_proc, s);
+	proc_handler_add(ph, "void get_duration(out int duration)",
+			get_duration, s);
+	proc_handler_add(ph, "void get_nb_frames(out int num_frames)",
+			get_nb_frames, s);
 
 	ffmpeg_source_update(s, settings);
 	return s;
