@@ -1,5 +1,7 @@
 #include <obs-module.h>
 
+#include "obs-outputs-config.h"
+
 #ifdef _WIN32
 #define WIN32_LEAN_AND_MEAN
 #include <winsock2.h>
@@ -11,6 +13,9 @@ OBS_MODULE_USE_DEFAULT_LOCALE("obs-outputs", "en-US")
 extern struct obs_output_info rtmp_output_info;
 extern struct obs_output_info null_output_info;
 extern struct obs_output_info flv_output_info;
+#if COMPILE_FTL
+extern struct obs_output_info ftl_output_info;
+#endif
 
 bool obs_module_load(void)
 {
@@ -22,6 +27,9 @@ bool obs_module_load(void)
 	obs_register_output(&rtmp_output_info);
 	obs_register_output(&null_output_info);
 	obs_register_output(&flv_output_info);
+#if COMPILE_FTL
+	obs_register_output(&ftl_output_info);
+#endif
 	return true;
 }
 
